@@ -1,4 +1,5 @@
 import hashlib
+import importlib.metadata
 import os
 import platform
 import re
@@ -15,11 +16,6 @@ from _pytest.outcomes import fail
 from _pytest.runner import runtestprotocol
 from packaging.version import parse as parse_version
 
-if sys.version_info >= (3, 8):
-    import importlib.metadata as importlib_metadata
-else:
-    import importlib_metadata
-
 try:
     from xdist.newhooks import pytest_handlecrashitem
 
@@ -39,8 +35,8 @@ def works_with_current_xdist():
 
     """
     try:
-        d = importlib_metadata.distribution("pytest-xdist")
-    except importlib_metadata.PackageNotFoundError:
+        d = importlib.metadata.distribution("pytest-xdist")
+    except importlib.metadata.PackageNotFoundError:
         return None
     else:
         return parse_version(d.version) >= parse_version("1.20")
@@ -188,7 +184,7 @@ def evaluate_condition(item, mark, condition: object) -> bool:
         try:
             filename = f"<{mark.name} condition>"
             condition_code = compile(condition, filename, "eval")
-            result = eval(condition_code, globals_)
+            result = eval(condition_code, globals_)  # noqa: S307
         except SyntaxError as exc:
             msglines = [
                 "Error evaluating %r condition" % mark.name,
@@ -365,9 +361,7 @@ class StatusDB:
 
     def _hash(self, crashitem: str) -> str:
         if crashitem not in self.hmap:
-            self.hmap[crashitem] = hashlib.sha1(
-                crashitem.encode(),
-            ).hexdigest()[:10]
+            self.hmap[crashitem] = hashlib.sha1(crashitem.encode()).hexdigest()[:10]  # noqa: S324
 
         return self.hmap[crashitem]
 
@@ -487,7 +481,15 @@ def pytest_runtest_teardown(item, nextitem):
         return
 
     _test_failed_statuses = getattr(item, "_test_failed_statuses", {})
-    if item.execution_count <= reruns and any(_test_failed_statuses.values()):
+
+    # Only remove non-function level actions from the stack if the test is to be re-run
+    # Exceeding re-run limits, being free of failue statuses, and encountering
+    # allowable exceptions indicate that the test is not to be re-ran.
+    if (
+        item.execution_count <= reruns
+        and any(_test_failed_statuses.values())
+        and not any(item._terminal_errors.values())
+    ):
         # clean cashed results from any level of setups
         _remove_cached_results_from_failed_fixtures(item)
 
@@ -504,10 +506,16 @@ def pytest_runtest_makereport(item, call):
     if result.when == "setup":
         # clean failed statuses at the beginning of each test/rerun
         setattr(item, "_test_failed_statuses", {})
+
+        # create a dict to store error-check results for each stage
+        setattr(item, "_terminal_errors", {})
+
     _test_failed_statuses = getattr(item, "_test_failed_statuses", {})
     _test_failed_statuses[result.when] = result.failed
     item._test_failed_statuses = _test_failed_statuses
 
+    item._terminal_errors[result.when] = _should_hard_fail_on_error(item, result)
+
 
 def pytest_runtest_protocol(item, nextitem):
     """
